#! /bin/bash
#
# Setup the build and test environment for SSL (for now)
#
# If you are running tests manually, then before running tests
# you need to:
# 1. Use BASH (if you are using csh)
# 2. "source" this file (e.g. "source env.sh")

[ "${__ACL_ENV_SH_SOURCED-}" ] && return 0

function __fi_error {
    echo $0: $*
    echo $0: aborting setup...
}

_TOPDIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && cd .. && pwd )

[ ! -f $_TOPDIR/makefile.top ] &&
    __fi_error cannot find $_TOPDIR/makefile.top &&
    return 1

source $_TOPDIR/makefile.top
# if on 64-bit, then SIXTYFOURBIT=yes
# always sets MACHINE and OS_NAME

[ ! "${MACHINE-}" ] && __fi_error MACHINE not set && return 1
[ ! "${OS_NAME-}" ] && __fi_error OS_NAME not set && return 1

function __acl_add_path {
    local var=$1
    local val=$2
    local temp

    if [ "${ACL_BUILD_OPENSSL_DIR-}" ]; then
	# if $val ends in /lib or /bin, use the same thing for
	# this one:
	val="$ACL_BUILD_OPENSSL_DIR/$(basename $val)"
    fi

    temp="$var=\${$var-.}:$val"
    #echo $temp
    eval "$temp"
    eval "export $var"
}

case $OS_NAME in
    aix)
	[ "$MACHINE" != "rs6000" ] && __fi_error aix: bad MACHINE: $MACHINE &&
	    return 1
	if [ "${SIXTYFOURBIT-}" = "yes" ]; then
	    ACL_OPENSSL_DIR=/usr/fi/openssl-prod.64
	else
	    if [ -d /usr/fi/openssl-prod.32 ]; then
		ACL_OPENSSL_DIR=/usr/fi/openssl-prod.32
	    else
		ACL_OPENSSL_DIR=/usr/fi/openssl-prod
	    fi
	fi
	__acl_add_path LD_LIBRARY_PATH $ACL_OPENSSL_DIR/lib
	;;

    darwin)
	[ "$MACHINE" != "x86" ] && [ "$MACHINE" != "x86_64" ] &&
	    __fi_error darwin: bad MACHINE: $MACHINE &&
	    return 1
	if [ "${SIXTYFOURBIT-}" = "yes" ]; then
	    ACL_OPENSSL_DIR=/opt/local
	else
	    ACL_OPENSSL_DIR=/opt/local
	fi
	# Switch from DYLD_LIBRARY_PATH to LD_LIBRARY_PATH because
	# I could not figure out how to get the DYLD_ version to work.
	# It seems the LD_ version is good enough for what we do,
	# dynamic loading at runtime.  The DYLD_ version messes with
	# the startup of the lisp executable itself, and the mess seemed
	# intractable.
	__acl_add_path LD_LIBRARY_PATH $ACL_OPENSSL_DIR/lib
	;;

    freebsd)
	[ "$MACHINE" != "x86" ] && [ "$MACHINE" != "x86_64" ] && 
	    __fi_error freebsd: bad MACHINE: $MACHINE &&
	    return 1
	ACL_OPENSSL_DIR=/usr/fi/openssl-prod
	__acl_add_path LD_LIBRARY_PATH $ACL_OPENSSL_DIR/lib
	;;

    linux)
	case $MACHINE in
	    x86|x86_64|arm64)
		ACL_OPENSSL_DIR=/usr/fi/openssl-prod
		__acl_add_path LD_LIBRARY_PATH $ACL_OPENSSL_DIR/lib
		;;
	    *)  __fi_error linux: bad MACHINE: $MACHINE && return 1
		;;
	esac
	;;

    sunos)
	case $MACHINE in
	    sparc)
		if [ "${SIXTYFOURBIT-}" = "yes" ]; then
		    ACL_OPENSSL_DIR=/usr/fi/sparcv9/openssl-prod
		else
		    ACL_OPENSSL_DIR=/usr/fi/openssl-prod
		fi
		;;
	    x86_64)
		if [ -d /usr/fi/amd64/openssl-prod ]; then
		    ACL_OPENSSL_DIR=/usr/fi/amd64/openssl-prod
		elif [ -d /usr/fi/openssl-prod.64 ]; then
		    ACL_OPENSSL_DIR=/usr/fi/openssl-prod.64
		else
		    __fi_error sunos: cannot find openssl && return 1
		fi
		;;
	    *)  __fi_error sunos: bad MACHINE: $MACHINE && return 1
		;;
	esac
	__acl_add_path PATH $ACL_OPENSSL_DIR/bin
	__acl_add_path LD_LIBRARY_PATH $ACL_OPENSSL_DIR/lib
	;;

    windows) # handled in bin/windows_*.env
	;;

    *)  __fi_error ounexpected OS_NAME: $OS_NAME && return 1
	;;
esac

# Allows script to be sourced multiple times without harm
__ACL_ENV_SH_SOURCED=yes

#echo ACL_OPENSSL_DIR=$ACL_OPENSSL_DIR

# Used by makefile.m to build the aclssl shared library
export ACL_OPENSSL_DIR
